def about():
    print("This is a simple Python package to interract with Galigeo's products")
    print("Author: Jules Pierrat")
    print("License: Apache License 2.0")
    print("Code source: https://github.com/JPTGGO/galigeopy")
    print("Documentation: https://galigeopy.readthedocs.io/en/latest/")

def check():
    return True